@@ -9,6 +9,8 @@
 
 #include "npy_config.h"
 
+#include "numpy/npy_3kcompat.h"
+
 #include "common.h"
 #include "scalartypes.h"
 #include "mapping.h"
@@ -58,7 +60,7 @@ PyArray_CastToType(PyArrayObject *mp, PyArray_Descr *at, int fortran)
         }
     }
 
-    out = PyArray_NewFromDescr(mp->ob_type, at,
+    out = PyArray_NewFromDescr(Py_TYPE(mp), at,
                                mp->nd,
                                mp->dimensions,
                                NULL, NULL,
@@ -100,15 +102,33 @@ PyArray_GetCastFunc(PyArray_Descr *descr, int type_num)
             key = PyInt_FromLong(type_num);
             cobj = PyDict_GetItem(obj, key);
             Py_DECREF(key);
-            if (PyCObject_Check(cobj)) {
-                castfunc = PyCObject_AsVoidPtr(cobj);
+            if (NpyCapsule_Check(cobj)) {
+                castfunc = NpyCapsule_AsVoidPtr(cobj);
             }
         }
-        if (castfunc) {
-            return castfunc;
-        }
     }
-    else {
+    if (PyTypeNum_ISCOMPLEX(descr->type_num) &&
+        !PyTypeNum_ISCOMPLEX(type_num) &&
+        PyTypeNum_ISNUMBER(type_num) &&
+        !PyTypeNum_ISBOOL(type_num)) {
+        PyObject *cls = NULL, *obj = NULL;
+        obj = PyImport_ImportModule("numpy.core");
+        if (obj) {
+            cls = PyObject_GetAttrString(obj, "ComplexWarning");
+            Py_DECREF(obj);
+        }
+#if PY_VERSION_HEX >= 0x02050000
+        PyErr_WarnEx(cls,
+                     "Casting complex values to real discards the imaginary "
+                     "part", 0);
+#else
+        PyErr_Warn(cls,
+                   "Casting complex values to real discards the imaginary "
+                   "part");
+#endif
+        Py_XDECREF(cls);
+    }
+    if (castfunc) {
         return castfunc;
     }
 
@@ -251,7 +271,7 @@ _broadcast_cast(PyArrayObject *out, PyArrayObject *in,
     if (PyDataType_REFCHK(in->descr)) {
         obptr = buffers[1];
         for (i = 0; i < N; i++, obptr+=selsize) {
-            PyArray_Item_XDECREF(obptr, out->descr);
+            PyArray_Item_XDECREF(obptr, in->descr);
         }
     }
     if (PyDataType_REFCHK(out->descr)) {
@@ -510,10 +530,6 @@ PyArray_CanCastSafely(int fromtype, int totype)
     if (totype == PyArray_BOOL) {
         return 0;
     }
-    if (fromtype == PyArray_DATETIME || fromtype == PyArray_TIMEDELTA ||
-            totype == PyArray_DATETIME || totype == PyArray_TIMEDELTA) {
-        return 0;
-    }
     if (totype == PyArray_OBJECT || totype == PyArray_VOID) {
         return 1;
     }
@@ -614,13 +630,23 @@ PyArray_CanCastSafely(int fromtype, int totype)
         if (PyTypeNum_ISCOMPLEX(totype)) {
             return (telsize >> 1) >= felsize;
         }
+        else if (PyTypeNum_ISFLOAT(totype) && (telsize == felsize)) {
+            /* On some systems, double == longdouble */
+            return 1;
+        }
         else {
             return totype > fromtype;
         }
     case PyArray_CFLOAT:
     case PyArray_CDOUBLE:
     case PyArray_CLONGDOUBLE:
-        return totype > fromtype;
+        if (PyTypeNum_ISCOMPLEX(totype) && (telsize == felsize)) {
+            /* On some systems, double == longdouble */
+            return 1;
+        }
+        else {
+            return totype > fromtype;
+        }
     case PyArray_STRING:
     case PyArray_UNICODE:
         return totype > fromtype;
