@@ -6,8 +6,13 @@ import copy
 from os import path
 from numpy.testing import *
 from numpy.testing.utils import _assert_valid_refcount
+from numpy.compat import asbytes, asunicode, asbytes_nested
 import numpy as np
 
+if sys.version_info[0] >= 3:
+    import io
+    StringIO = io.BytesIO
+
 rlevel = 1
 
 class TestRegression(TestCase):
@@ -79,7 +84,7 @@ class TestRegression(TestCase):
         b = a[:,:2,]
         def rs():
             b.shape = (10,)
-        self.failUnlessRaises(AttributeError,rs)
+        self.assertRaises(AttributeError,rs)
 
     def test_bool(self,level=rlevel):
         """Ticket #60"""
@@ -128,17 +133,21 @@ class TestRegression(TestCase):
 
     def test_mem_dtype_align(self,level=rlevel):
         """Ticket #93"""
-        self.failUnlessRaises(TypeError,np.dtype,
+        self.assertRaises(TypeError,np.dtype,
                               {'names':['a'],'formats':['foo']},align=1)
 
+    @dec.knownfailureif(sys.version_info[0] >= 3 or (
+                        sys.platform == 'win32' and '64 bit' in sys.version),
+                        "numpy.intp('0xff', 16) not supported on Py3, "
+                        "as it does not inherit from Python int")
     def test_intp(self,level=rlevel):
         """Ticket #99"""
         i_width = np.int_(0).nbytes*2 - 1
-        long('0x' + 'f'*i_width,16)
-        #self.failUnlessRaises(OverflowError,np.intp,'0x' + 'f'*(i_width+1),16)
-        #self.failUnlessRaises(ValueError,np.intp,'0x1',32)
-        assert_equal(255,np.long('0xFF',16))
-        assert_equal(1024,np.long(1024))
+        np.intp('0x' + 'f'*i_width,16)
+        self.assertRaises(OverflowError,np.intp,'0x' + 'f'*(i_width+1),16)
+        self.assertRaises(ValueError,np.intp,'0x1',32)
+        assert_equal(255,np.intp('0xFF',16))
+        assert_equal(1024,np.intp(1024))
 
     def test_endian_bool_indexing(self,level=rlevel):
         """Ticket #105"""
@@ -191,7 +200,7 @@ class TestRegression(TestCase):
         """Ticket #128"""
         x = np.arange(9).reshape((3,3))
         y = np.array([0,0,0])
-        self.failUnlessRaises(ValueError,np.hstack,(x,y))
+        self.assertRaises(ValueError,np.hstack,(x,y))
 
     def test_squeeze_type(self,level=rlevel):
         """Ticket #133"""
@@ -241,8 +250,8 @@ class TestRegression(TestCase):
         x = np.empty((3,1))
         def bfa(): x[:] = np.arange(3)
         def bfb(): x[:] = np.arange(3,dtype=float)
-        self.failUnlessRaises(ValueError, bfa)
-        self.failUnlessRaises(ValueError, bfb)
+        self.assertRaises(ValueError, bfa)
+        self.assertRaises(ValueError, bfb)
 
     def test_unpickle_dtype_with_object(self,level=rlevel):
         """Implemented in r2840"""
@@ -258,7 +267,7 @@ class TestRegression(TestCase):
         """Ticket #196"""
         dt = np.dtype([('x',int),('y',np.object_)])
         # Wrong way
-        self.failUnlessRaises(ValueError, np.array, [1,'object'], dt)
+        self.assertRaises(ValueError, np.array, [1,'object'], dt)
         # Correct way
         np.array([(1,'object')],dt)
 
@@ -274,14 +283,14 @@ class TestRegression(TestCase):
         """Ticket #205"""
         tmp = np.array([])
         def index_tmp(): tmp[np.array(10)]
-        self.failUnlessRaises(IndexError, index_tmp)
+        self.assertRaises(IndexError, index_tmp)
 
     def test_chararray_rstrip(self,level=rlevel):
         """Ticket #222"""
         x = np.chararray((1,),5)
-        x[0] = 'a   '
+        x[0] = asbytes('a   ')
         x = x.rstrip()
-        assert_equal(x[0], 'a')
+        assert_equal(x[0], asbytes('a'))
 
     def test_object_array_shape(self,level=rlevel):
         """Ticket #239"""
@@ -433,11 +442,11 @@ class TestRegression(TestCase):
 
     def test_numeric_carray_compare(self, level=rlevel):
         """Ticket #341"""
-        assert_equal(np.array([ 'X' ], 'c'),'X')
+        assert_equal(np.array(['X'], 'c'), asbytes('X'))
 
     def test_string_array_size(self, level=rlevel):
         """Ticket #342"""
-        self.failUnlessRaises(ValueError,
+        self.assertRaises(ValueError,
                               np.array,[['X'],['X','X','X']],'|S1')
 
     def test_dtype_repr(self, level=rlevel):
@@ -508,8 +517,8 @@ class TestRegression(TestCase):
 
     def test_convolve_empty(self, level=rlevel):
         """Convolve should raise an error for empty input array."""
-        self.failUnlessRaises(ValueError,np.convolve,[],[1])
-        self.failUnlessRaises(ValueError,np.convolve,[1],[])
+        self.assertRaises(ValueError,np.convolve,[],[1])
+        self.assertRaises(ValueError,np.convolve,[1],[])
 
     def test_multidim_byteswap(self, level=rlevel):
         """Ticket #449"""
@@ -524,8 +533,8 @@ class TestRegression(TestCase):
 
     def test_junk_in_string_fields_of_recarray(self, level=rlevel):
         """Ticket #483"""
-        r = np.array([['abc']], dtype=[('var1', '|S20')])
-        assert str(r['var1'][0][0]) == 'abc'
+        r = np.array([[asbytes('abc')]], dtype=[('var1', '|S20')])
+        assert asbytes(r['var1'][0][0]) == asbytes('abc')
 
     def test_take_output(self, level=rlevel):
         """Ensure that 'take' honours output parameter."""
@@ -587,7 +596,7 @@ class TestRegression(TestCase):
 
     def test_mem_on_invalid_dtype(self):
         "Ticket #583"
-        self.failUnlessRaises(ValueError, np.fromiter, [['12',''],['13','']], str)
+        self.assertRaises(ValueError, np.fromiter, [['12',''],['13','']], str)
 
     def test_dot_negative_stride(self, level=rlevel):
         """Ticket #588"""
@@ -602,7 +611,7 @@ class TestRegression(TestCase):
             x = np.ones([484,286])
             y = np.zeros([484,286])
             x |= y
-        self.failUnlessRaises(TypeError,rs)
+        self.assertRaises(TypeError,rs)
 
     def test_unicode_scalar(self, level=rlevel):
         """Ticket #600"""
@@ -624,7 +633,7 @@ class TestRegression(TestCase):
         s = np.ones(10,dtype=float)
         x = np.array((15,),dtype=float)
         def ia(x,s): x[(s>0)]=1.0
-        self.failUnlessRaises(ValueError,ia,x,s)
+        self.assertRaises(ValueError,ia,x,s)
 
     def test_mem_scalar_indexing(self, level=rlevel):
         """Ticket #603"""
@@ -793,7 +802,7 @@ class TestRegression(TestCase):
         """Ticket 702"""
         class MyFloat:
             def __float__(self):
-                return 1
+                return 1.0
 
         tmp = np.atleast_1d([MyFloat()])
         tmp2 = tmp.astype(float)
@@ -815,7 +824,7 @@ class TestRegression(TestCase):
 
     def test_mem_fromiter_invalid_dtype_string(self, level=rlevel):
         x = [1,2,3]
-        self.failUnlessRaises(ValueError,
+        self.assertRaises(ValueError,
                               np.fromiter, [xi for xi in x], dtype='S')
 
     def test_reduce_big_object_array(self, level=rlevel):
@@ -876,7 +885,10 @@ class TestRegression(TestCase):
         # The dtype is float64, but the isbuiltin attribute is 0.
         data_dir = path.join(path.dirname(__file__), 'data')
         filename = path.join(data_dir, "astype_copy.pkl")
-        xp = pickle.load(open(filename))
+        if sys.version_info[0] >= 3:
+            xp = pickle.load(open(filename, 'rb'), encoding='latin1')
+        else:
+            xp = pickle.load(open(filename))
         xpd = xp.astype(np.float64)
         assert (xp.__array_interface__['data'][0] !=
                 xpd.__array_interface__['data'][0])
@@ -980,7 +992,7 @@ class TestRegression(TestCase):
 
     def test_char_array_creation(self, level=rlevel):
         a = np.array('123', dtype='c')
-        b = np.array(['1','2','3'])
+        b = np.array(asbytes_nested(['1','2','3']))
         assert_equal(a,b)
 
     def test_unaligned_unicode_access(self, level=rlevel) :
@@ -988,13 +1000,20 @@ class TestRegression(TestCase):
         for i in range(1,9) :
             msg = 'unicode offset: %d chars'%i
             t = np.dtype([('a','S%d'%i),('b','U2')])
-            x = np.array([('a',u'b')], dtype=t)
-            assert_equal(str(x), "[('a', u'b')]", err_msg=msg)
+            x = np.array([(asbytes('a'),u'b')], dtype=t)
+            if sys.version_info[0] >= 3:
+                assert_equal(str(x), "[(b'a', 'b')]", err_msg=msg)
+            else:
+                assert_equal(str(x), "[('a', u'b')]", err_msg=msg)
 
     def test_sign_for_complex_nan(self, level=rlevel):
         """Ticket 794."""
         C = np.array([-np.inf, -2+1j, 0, 2-1j, np.inf, np.nan])
-        have = np.sign(C)
+        err = np.seterr(invalid="ignore")
+        try:
+            have = np.sign(C)
+        finally:
+            np.seterr(**err)        
         want = np.array([-1+0j, -1+0j, 0+0j, 1+0j, 1+0j, np.nan])
         assert_equal(have, want)
 
@@ -1022,10 +1041,15 @@ class TestRegression(TestCase):
         assert c.dtype == object
         assert d.dtype == object
 
+    def test_array_resize_method_system_error(self):
+        """Ticket #840 - order should be an invalid keyword."""
+        x = np.array([[0,1],[2,3]])
+        self.assertRaises(TypeError, x.resize, (2,2), order='C')
+
     def test_for_zero_length_in_choose(self, level=rlevel):
         "Ticket #882"
         a = np.array(1)
-        self.failUnlessRaises(ValueError, lambda x: x.choose([]), a)
+        self.assertRaises(ValueError, lambda x: x.choose([]), a)
 
     def test_array_ndmin_overflow(self):
         "Ticket #947."
@@ -1033,15 +1057,19 @@ class TestRegression(TestCase):
 
     def test_errobj_reference_leak(self, level=rlevel):
         """Ticket #955"""
-        z = int(0)
-        p = np.int32(-1)
+        old_err = np.seterr(all="ignore")
+        try:
+            z = int(0)
+            p = np.int32(-1)
 
-        gc.collect()
-        n_before = len(gc.get_objects())
-        z**p  # this shouldn't leak a reference to errobj
-        gc.collect()
-        n_after = len(gc.get_objects())
-        assert n_before >= n_after, (n_before, n_after)
+            gc.collect()
+            n_before = len(gc.get_objects())
+            z**p  # this shouldn't leak a reference to errobj
+            gc.collect()
+            n_after = len(gc.get_objects())
+            assert n_before >= n_after, (n_before, n_after)
+        finally:
+            np.seterr(**old_err)
 
     def test_void_scalar_with_titles(self, level=rlevel):
         """No ticket"""
@@ -1087,7 +1115,7 @@ class TestRegression(TestCase):
         good = 'Maximum allowed size exceeded'
         try:
             a = np.arange(sz)
-            self.failUnless(np.size == sz)
+            self.assertTrue(np.size == sz)
         except ValueError, e:
             if not str(e) == good:
                 self.fail("Got msg '%s', expected '%s'" % (e, good))
@@ -1111,6 +1139,12 @@ class TestRegression(TestCase):
         t = ((1,), np.array(1))
         assert_raises(ValueError, lambda: np.array(t))
 
+    @dec.knownfailureif(True, "Fix this for 1.5.0.")
+    def test_array_from_sequence_scalar_array2(self):
+        """Ticket #1081: weird array with strange input..."""
+        t = np.array([np.array([]), np.array(0, object)])
+        assert_raises(ValueError, lambda: np.array(t))
+
     def test_array_too_big(self):
         """Ticket #1080."""
         assert_raises(ValueError, np.zeros, [2**10]*10)
@@ -1132,13 +1166,13 @@ class TestRegression(TestCase):
     def test_object_array_to_fixed_string(self):
         """Ticket #1235."""
         a = np.array(['abcdefgh', 'ijklmnop'], dtype=np.object_)
-        b = np.array(a, dtype=(np.string_, 8))
+        b = np.array(a, dtype=(np.str_, 8))
         assert_equal(a, b)
-        c = np.array(a, dtype=(np.string_, 5))
+        c = np.array(a, dtype=(np.str_, 5))
         assert_equal(c, np.array(['abcde', 'ijklm']))
-        d = np.array(a, dtype=(np.string_, 12))
+        d = np.array(a, dtype=(np.str_, 12))
         assert_equal(a, d)
-        e = np.empty((2, ), dtype=(np.string_, 8))
+        e = np.empty((2, ), dtype=(np.str_, 8))
         e[:] = a[:]
         assert_equal(a, e)
 
@@ -1147,7 +1181,7 @@ class TestRegression(TestCase):
         a = np.array([[u'abc', u'\u03a3'], [u'asdf', u'erw']], dtype='U')
         def fail():
             b = np.array(a, 'S4')
-        self.failUnlessRaises(UnicodeEncodeError, fail)
+        self.assertRaises(UnicodeEncodeError, fail)
 
     def test_mixed_string_unicode_array_creation(self):
         a = np.array(['1234', u'123'])
@@ -1202,7 +1236,7 @@ class TestRegression(TestCase):
             # big-endian machine
             assert_equal(x, np.fromstring(y.tostring(), dtype='<c8'))
 
-    def test_structured_arrays_with_objects(self):
+    def test_structured_arrays_with_objects1(self):
         """Ticket #1299"""
         stra = 'aaaa'
         strb = 'bbbb'
@@ -1220,21 +1254,128 @@ class TestRegression(TestCase):
         x[x.nonzero()] = x.ravel()[:1]
         assert sys.getrefcount(strb) == numb
         assert sys.getrefcount(stra) == numa + 2
-        
+
+    def test_duplicate_title_and_name(self):
+        """Ticket #1254"""
+        def func():
+            x = np.dtype([(('a', 'a'), 'i'), ('b', 'i')])
+        self.assertRaises(ValueError, func)
+
     def test_signed_integer_division_overflow(self):
         """Ticket #1317."""
         def test_type(t):
             min = np.array([np.iinfo(t).min])
             min /= -1
 
-        for t in (np.int8, np.int16, np.int32, np.int64, np.int, np.long):
-            test_type(t)
+        old_err = np.seterr(divide="ignore")
+        try:
+            for t in (np.int8, np.int16, np.int32, np.int64, np.int, np.long):
+                test_type(t)
+        finally:
+            np.seterr(**old_err)
 
+    def test_buffer_hashlib(self):
+        try:
+            from hashlib import md5
+        except ImportError:
+            from md5 import new as md5
+
+        x = np.array([1,2,3], dtype=np.dtype('<i4'))
+        assert_equal(md5(x).hexdigest(), '2a1dd1e1e59d0a384c26951e316cd7e6')
+
+    def test_numeric_handleError(self):
+        """Ticket #1405"""
+        from numpy import numarray
+        # Just make sure this doesn't throw an exception
+        numarray.handleError(0, "")
+
+    def test_0d_string_scalar(self):
+        # Bug #1436; the following should succeed
+        np.asarray('x', '>c')
+
+    def test_log1p_compiler_shenanigans(self):
+        # Check if log1p is behaving on 32 bit intel systems.
+        assert_(np.isfinite(np.log1p(np.exp2(-53))))
+
+    def test_fromiter_comparison(self, level=rlevel):
+        a = np.fromiter(range(10), dtype='b')
+        b = np.fromiter(range(10), dtype='B')
+        assert np.alltrue(a == np.array([0,1,2,3,4,5,6,7,8,9]))
+        assert np.alltrue(b == np.array([0,1,2,3,4,5,6,7,8,9]))
+
+    def test_fromstring_crash(self):
+        # Ticket #1345: the following should not cause a crash
+        np.fromstring(asbytes('aa, aa, 1.0'), sep=',')
+
+    def test_ticket_1539(self):
+        dtypes = [x for x in np.typeDict.values()
+                  if issubclass(x, np.number)]
+        a = np.array([], dtypes[0])
+        failures = []
+        for x in dtypes:
+            b = a.astype(x)
+            for y in dtypes:
+                c = a.astype(y)
+                try:
+                    np.dot(b, c)
+                except TypeError, e:
+                    failures.append((x, y))
+        if failures:
+            raise AssertionError("Failures: %r" % failures)
+
+    def test_ticket_1538(self):
+        x = np.finfo(np.float32)
+        for name in 'eps epsneg max min resolution tiny'.split():
+            assert_equal(type(getattr(x, name)), np.float32,
+                         err_msg=name)
+
+    def test_ticket_1434(self):
+        # Check that the out= argument in var and std has an effect
+        data = np.array(((1,2,3),(4,5,6),(7,8,9)))
+        out = np.zeros((3,))
+
+        ret = data.var(axis=1, out=out)
+        assert_(ret is out)
+        assert_array_equal(ret, data.var(axis=1))
+
+        ret = data.std(axis=1, out=out)
+        assert_(ret is out)
+        assert_array_equal(ret, data.std(axis=1))
+
+    def test_complex_nan_maximum(self):
+        cnan = complex(0, np.nan)
+        assert_equal(np.maximum(1, cnan), cnan)
+
+    def test_subclass_int_tuple_assignment(self):
+        # ticket #1563
+        class Subclass(np.ndarray):
+            def __new__(cls,i):
+                return np.ones((i,)).view(cls)
+        x = Subclass(5)
+        x[(0,)] = 2 # shouldn't raise an exception
+        assert_equal(x[0], 2)
+
+    def test_ufunc_no_unnecessary_views(self):
+        # ticket #1548
+        class Subclass(np.ndarray):
+            pass
+        x = np.array([1,2,3]).view(Subclass)
+        y = np.add(x, x, x)
+        assert_equal(id(x), id(y))
+
+    def test_take_refcount(self):
+        # ticket #939
+        a = np.arange(16, dtype=np.float)
+        a.shape = (4,4)
+        lut = np.ones((5 + 3, 4), np.float)
+        rgba = np.empty(shape=a.shape + (4,), dtype=lut.dtype)
+        c1 = sys.getrefcount(rgba)
+        try:
+            lut.take(a, axis=0, mode='clip', out=rgba)
+        except TypeError:
+            pass
+        c2 = sys.getrefcount(rgba)
+        assert_equal(c1, c2)
 
 if __name__ == "__main__":
     run_module_suite()
-    def test_duplicate_title_and_name(self):
-        """Ticket #1254"""
-        def func():
-            x = np.dtype([(('a', 'a'), 'i'), ('b', 'i')])
-        self.failUnlessRaises(ValueError, func)
