@@ -1,3 +1,5 @@
+import sys
+
 from numpy.testing import *
 import numpy.core.umath as ncu
 import numpy as np
@@ -6,9 +8,27 @@ import numpy as np
 # TODO: conj 'symmetry'
 # TODO: FPU exceptions
 
+# At least on Windows the results of many complex functions are not conforming
+# to the C99 standard. See ticket 1574.
+err = np.seterr(invalid='ignore')
+try:
+    functions_seem_flaky = (np.exp(complex(np.inf, 0)).imag != 0)
+finally:
+    np.seterr(**err)
+# TODO: replace with a check on whether platform-provided C99 funcs are used
+have_platform_functions = (sys.platform == 'win32')
+skip_complex_tests = have_platform_functions and functions_seem_flaky
+
+def platform_skip(func):
+    return dec.skipif(skip_complex_tests,
+        "Numpy is using complex functions (e.g. sqrt) provided by your"
+        "platform's C library. However, they do not seem to behave according"
+        "to C99 -- so C99 tests are skipped.")(func)
+
+
 class TestCexp(object):
     def test_simple(self):
-        check = check_complex_value 
+        check = check_complex_value
         f = np.exp
 
         yield check, f, 1, 0, np.exp(1), 0, False
@@ -17,10 +37,11 @@ class TestCexp(object):
         ref = np.exp(1) * np.complex(np.cos(1), np.sin(1))
         yield check, f, 1, 1, ref.real, ref.imag, False
 
+    @platform_skip
     def test_special_values(self):
         # C99: Section G 6.3.1
 
-        check = check_complex_value 
+        check = check_complex_value
         f = np.exp
 
         # cexp(+-0 + 0i) is 1 + 0i
@@ -52,38 +73,54 @@ class TestCexp(object):
 
         # cexp(-inf + inf i) is +-0 +- 0i (signs unspecified)
         def _check_ninf_inf(dummy):
-            z = f(np.array(np.complex(-np.inf, np.inf)))
-            if z.real != 0 or z.imag != 0:
-                raise AssertionError(
-                        "cexp(-inf, inf) is (%f, %f), expected (+-0, +-0)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, inf) is (%f, %f), expected (+-0, +-0)"
+            err = np.seterr(invalid='ignore')
+            try:
+                z = f(np.array(np.complex(-np.inf, np.inf)))
+                if z.real != 0 or z.imag != 0:
+                    raise AssertionError(msgform %(z.real, z.imag))
+            finally:
+                np.seterr(**err)
+
         yield _check_ninf_inf, None
 
         # cexp(inf + inf i) is +-inf + NaNi and raised invalid FPU ex.
         def _check_inf_inf(dummy):
-            z = f(np.array(np.complex(np.inf, np.inf)))
-            if not np.isinf(z.real) or not np.isnan(z.imag):
-                raise AssertionError(
-                        "cexp(inf, inf) is (%f, %f), expected (+-inf, nan)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(inf, inf) is (%f, %f), expected (+-inf, nan)"
+            err = np.seterr(invalid='ignore')
+            try:
+                z = f(np.array(np.complex(np.inf, np.inf)))
+                if not np.isinf(z.real) or not np.isnan(z.imag):
+                    raise AssertionError(msgform % (z.real, z.imag))
+            finally:
+                np.seterr(**err)
+
         yield _check_inf_inf, None
 
         # cexp(-inf + nan i) is +-0 +- 0i
         def _check_ninf_nan(dummy):
-            z = f(np.array(np.complex(-np.inf, np.nan)))
-            if z.real != 0 or z.imag != 0:
-                raise AssertionError(
-                        "cexp(-inf, nan) is (%f, %f), expected (+-0, +-0)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, nan) is (%f, %f), expected (+-0, +-0)"
+            err = np.seterr(invalid='ignore')
+            try:
+                z = f(np.array(np.complex(-np.inf, np.nan)))
+                if z.real != 0 or z.imag != 0:
+                    raise AssertionError(msgform % (z.real, z.imag))
+            finally:
+                np.seterr(**err)
+
         yield _check_ninf_nan, None
 
         # cexp(inf + nan i) is +-inf + nan
         def _check_inf_nan(dummy):
-            z = f(np.array(np.complex(np.inf, np.nan)))
-            if not np.isinf(z.real) or not np.isnan(z.imag):
-                raise AssertionError(
-                        "cexp(-inf, nan) is (%f, %f), expected (+-inf, nan)" \
-                        % (z.real, z.imag))
+            msgform = "cexp(-inf, nan) is (%f, %f), expected (+-inf, nan)"
+            err = np.seterr(invalid='ignore')
+            try:
+                z = f(np.array(np.complex(np.inf, np.nan)))
+                if not np.isinf(z.real) or not np.isnan(z.imag):
+                    raise AssertionError(msgform % (z.real, z.imag))
+            finally:
+                np.seterr(**err)
+
         yield _check_inf_nan, None
 
         # cexp(nan + yi) is nan + nani for y != 0 (optional: raises invalid FPU
@@ -111,26 +148,47 @@ class TestClog(TestCase):
         for i in range(len(x)):
             assert_almost_equal(y[i], y_r[i])
 
+    #@platform_skip
+    # Clog tests have been failing for a long time on Windows, so always skip
+    # on that platform. In 1.5.x the platform_skip decorator is Windows-only
+    # anyway.
+    @dec.knownfailureif(sys.platform == 'win32',
+                        "clog(- inf + i inf) fails on Windows.")
     def test_special_values(self):
         xl = []
         yl = []
 
         # From C99 std (Sec 6.3.2)
         # XXX: check exceptions raised
+        # --- raise for invalid fails.
 
         # clog(-0 + i0) returns -inf + i pi and raises the 'divide-by-zero'
         # floating-point exception.
-        x = np.array([np.NZERO], dtype=np.complex)
-        y = np.complex(-np.inf, np.pi)
-        assert_almost_equal(np.log(x), y)
+        err = np.seterr(divide='raise')
+        try:
+            x = np.array([np.NZERO], dtype=np.complex)
+            y = np.complex(-np.inf, np.pi)
+            self.assertRaises(FloatingPointError, np.log, x)
+            np.seterr(divide='ignore')
+            assert_almost_equal(np.log(x), y)
+        finally:
+            np.seterr(**err)
+
         xl.append(x)
         yl.append(y)
 
         # clog(+0 + i0) returns -inf + i0 and raises the 'divide-by-zero'
         # floating-point exception.
-        x = np.array([0], dtype=np.complex)
-        y = np.complex(-np.inf, 0)
-        assert_almost_equal(np.log(x), y)
+        err = np.seterr(divide='raise')
+        try:
+            x = np.array([0], dtype=np.complex)
+            y = np.complex(-np.inf, 0)
+            self.assertRaises(FloatingPointError, np.log, x)
+            np.seterr(divide='ignore')
+            assert_almost_equal(np.log(x), y)
+        finally:
+            np.seterr(**err)
+
         xl.append(x)
         yl.append(y)
 
@@ -148,14 +206,28 @@ class TestClog(TestCase):
 
         # clog(x + iNaN) returns NaN + iNaN and optionally raises the
         # 'invalid' floating- point exception, for finite x.
-        x = np.array([complex(1., np.nan)], dtype=np.complex)
-        y = np.complex(np.nan, np.nan)
-        assert_almost_equal(np.log(x), y)
+        err = np.seterr(invalid='raise')
+        try:
+            x = np.array([complex(1., np.nan)], dtype=np.complex)
+            y = np.complex(np.nan, np.nan)
+            #self.assertRaises(FloatingPointError, np.log, x)
+            np.seterr(invalid='ignore')
+            assert_almost_equal(np.log(x), y)
+        finally:
+            np.seterr(**err)
+
         xl.append(x)
         yl.append(y)
 
-        x = np.array([np.inf + np.nan * 1j], dtype=np.complex)
-        assert_almost_equal(np.log(x), y)
+        err = np.seterr(invalid='raise')
+        try:
+            x = np.array([np.inf + 1j * np.nan], dtype=np.complex)
+            #self.assertRaises(FloatingPointError, np.log, x)
+            np.seterr(invalid='ignore')
+            assert_almost_equal(np.log(x), y)
+        finally:
+            np.seterr(**err)
+
         xl.append(x)
         yl.append(y)
 
@@ -176,7 +248,11 @@ class TestClog(TestCase):
         # clog(- inf + i inf) returns +inf + i3pi /4.
         x = np.array([complex(-np.inf, np.inf)], dtype=np.complex)
         y = np.complex(np.inf, 0.75 * np.pi)
-        assert_almost_equal(np.log(x), y)
+        err = np.seterr(invalid="ignore")
+        try:
+            assert_almost_equal(np.log(x), y)
+        finally:
+            np.seterr(**err)
         xl.append(x)
         yl.append(y)
 
@@ -224,10 +300,15 @@ class TestClog(TestCase):
         # clog(conj(z)) = conj(clog(z)).
         xa = np.array(xl, dtype=np.complex)
         ya = np.array(yl, dtype=np.complex)
-        for i in range(len(xa)):
-            assert_almost_equal(np.log(np.conj(xa[i])), np.conj(np.log(xa[i])))
+        err = np.seterr(divide='ignore')
+        try:
+            for i in range(len(xa)):
+                assert_almost_equal(np.log(np.conj(xa[i])), np.conj(np.log(xa[i])))
+        finally:
+            np.seterr(**err)
 
 class TestCsqrt(object):
+
     def test_simple(self):
         # sqrt(1)
         yield check_complex_value, np.sqrt, 1, 0, 1, 0
@@ -247,8 +328,9 @@ class TestCsqrt(object):
     #def test_branch_cut(self):
     #    _check_branch_cut(f, -1, 0, 1, -1)
 
+    @platform_skip
     def test_special_values(self):
-        check = check_complex_value 
+        check = check_complex_value
         f = np.sqrt
 
         # C99: Sec G 6.4.2
@@ -281,11 +363,15 @@ class TestCsqrt(object):
 
         # csqrt(-inf + nani) is nan +- infi (both +i infi are valid)
         def _check_ninf_nan(dummy):
+            msgform = "csqrt(-inf, nan) is (%f, %f), expected (nan, +-inf)"
             z = np.sqrt(np.array(np.complex(-np.inf, np.nan)))
-            if not np.isnan(z.real) or not np.isinf(z.imag):
-                raise AssertionError(
-                        "csqrt(-inf, nan) is (%f, %f), expected (nan, +-inf)" \
-                        % (z.real, z.imag))
+            #Fixme: ugly workaround for isinf bug.
+            err = np.seterr(invalid='ignore')
+            try:
+                if not (np.isnan(z.real) and np.isinf(z.imag)):
+                    raise AssertionError(msgform % (z.real, z.imag))
+            finally:
+                np.seterr(**err)
 
         yield _check_ninf_nan, None
 
@@ -304,10 +390,14 @@ class TestCsqrt(object):
 class TestCpow(TestCase):
     def test_simple(self):
         x = np.array([1+1j, 0+2j, 1+2j, np.inf, np.nan])
-        y_r = x ** 2
-        y = np.power(x, 2)
-        for i in range(len(x)):
-            assert_almost_equal(y[i], y_r[i])
+        err = np.seterr(invalid='ignore')
+        try:
+            y_r = x ** 2
+            y = np.power(x, 2)
+            for i in range(len(x)):
+                assert_almost_equal(y[i], y_r[i])
+        finally:
+            np.seterr(**err)
 
     def test_scalar(self):
         x = np.array([1, 1j,         2,  2.5+.37j, np.inf, np.nan])
@@ -318,9 +408,13 @@ class TestCpow(TestCase):
         # Substitute a result allowed by C99 standard
         p_r[4] = complex(np.inf, np.nan)
         # Do the same with numpy complex scalars
-        n_r = [x[i] ** y[i] for i in lx]
-        for i in lx:
-            assert_almost_equal(n_r[i], p_r[i], err_msg='Loop %d\n' % i)
+        err = np.seterr(invalid='ignore')
+        try:
+            n_r = [x[i] ** y[i] for i in lx]
+            for i in lx:
+                assert_almost_equal(n_r[i], p_r[i], err_msg='Loop %d\n' % i)
+        finally:
+            np.seterr(**err)
 
     def test_array(self):
         x = np.array([1, 1j,         2,  2.5+.37j, np.inf, np.nan])
@@ -331,17 +425,25 @@ class TestCpow(TestCase):
         # Substitute a result allowed by C99 standard
         p_r[4] = complex(np.inf, np.nan)
         # Do the same with numpy arrays
-        n_r = x ** y
-        for i in lx:
-            assert_almost_equal(n_r[i], p_r[i], err_msg='Loop %d\n' % i)
+        err = np.seterr(invalid='ignore')
+        try:
+            n_r = x ** y
+            for i in lx:
+                assert_almost_equal(n_r[i], p_r[i], err_msg='Loop %d\n' % i)
+        finally:
+            np.seterr(**err)
 
 class TestCabs(object):
     def test_simple(self):
         x = np.array([1+1j, 0+2j, 1+2j, np.inf, np.nan])
         y_r = np.array([np.sqrt(2.), 2, np.sqrt(5), np.inf, np.nan])
+        err = np.seterr(invalid="ignore")
         y = np.abs(x)
-        for i in range(len(x)):
-            assert_almost_equal(y[i], y_r[i])
+        try:
+            for i in range(len(x)):
+                assert_almost_equal(y[i], y_r[i])
+        finally:
+            np.seterr(**err)
 
     def test_fabs(self):
         # Test that np.abs(x +- 0j) == np.abs(x) (as mandated by C99 for cabs)
@@ -352,7 +454,11 @@ class TestCabs(object):
         assert_array_equal(np.abs(x), np.real(x))
 
         x = np.array([complex(np.inf, np.NZERO)], dtype=np.complex)
-        assert_array_equal(np.abs(x), np.real(x))
+        err = np.seterr(invalid="ignore")
+        try:
+            assert_array_equal(np.abs(x), np.real(x))
+        finally:
+            np.seterr(**err)
 
         x = np.array([complex(np.nan, np.NZERO)], dtype=np.complex)
         assert_array_equal(np.abs(x), np.real(x))
@@ -465,12 +571,16 @@ def check_real_value(f, x1, y1, x, exact=True):
         assert_almost_equal(f(z1), x)
 
 def check_complex_value(f, x1, y1, x2, y2, exact=True):
+    err = np.seterr(invalid='ignore')
     z1 = np.array([complex(x1, y1)])
     z2 = np.complex(x2, y2)
-    if exact:
-        assert_equal(f(z1), z2)
-    else:
-        assert_almost_equal(f(z1), z2)
+    try:
+        if exact:
+            assert_equal(f(z1), z2)
+        else:
+            assert_almost_equal(f(z1), z2)
+    finally:
+        np.seterr(**err)
 
 if __name__ == "__main__":
     run_module_suite()
