@@ -11,13 +11,16 @@ Support code for building Python extensions on Windows.
 import os
 import subprocess
 import sys
-import log
 import subprocess
 import re
 
 # Overwrite certain distutils.ccompiler functions:
 import numpy.distutils.ccompiler
 
+if sys.version_info[0] < 3:
+    import log
+else:
+    from numpy.distutils import log
 # NT stuff
 # 1. Make sure libpython<version>.a exists for gcc.  If not, build it.
 # 2. Force windows to use gcc (we're struggling with MSVC and g77 support)
@@ -171,7 +174,10 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
             func = distutils.cygwinccompiler.CygwinCCompiler.link
         else:
             func = UnixCCompiler.link
-        func(*args[:func.im_func.func_code.co_argcount])
+        if sys.version_info[0] >= 3:
+            func(*args[:func.__code__.co_argcount])
+        else:
+            func(*args[:func.im_func.func_code.co_argcount])
         return
 
     def object_filenames (self,
@@ -192,9 +198,9 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
                 base = base[1:]
 
             if ext not in (self.src_extensions + ['.rc','.res']):
-                raise UnknownFileError, \
+                raise UnknownFileError(
                       "unknown file type '%s' (from '%s')" % \
-                      (ext, src_name)
+                      (ext, src_name))
             if strip_dir:
                 base = os.path.basename (base)
             if ext == '.res' or ext == '.rc':
@@ -212,7 +218,7 @@ class Mingw32CCompiler(distutils.cygwinccompiler.CygwinCCompiler):
 def find_python_dll():
     maj, min, micro = [int(i) for i in sys.version_info[:3]]
     dllname = 'python%d%d.dll' % (maj, min)
-    print "Looking for %s" % dllname
+    print ("Looking for %s" % dllname)
 
     # We can't do much here:
     # - find it in python main dir
@@ -433,10 +439,10 @@ def check_embedded_msvcr_match_linked(msver):
     if msvcv:
         maj = int(msvcv[5:6])
         if not maj == int(msver):
-            raise ValueError, \
+            raise ValueError(
                   "Discrepancy between linked msvcr " \
                   "(%d) and the one about to be embedded " \
-                  "(%d)" % (int(msver), maj)
+                  "(%d)" % (int(msver), maj))
 
 def configtest_name(config):
     base = os.path.basename(config._gen_temp_sourcefile("yo", [], "c"))
